package luento14;

import java.util.Arrays;

import fi.jyu.mit.graphics.*;
import fi.jyu.mit.ohj2.*;

/**
 * Ohjelmalla pelataan hirsipuu-peli.
 * 
 * Suunitelma:
 * <pre>
 * Peli:    Tervetuloa pelaamaan Hirsipuupeli!
 *
 * Peli:    Anna nimesi:
 * Pelaaja:Jarkko <ret>
 *
 * Peli:    Terve Jarkko!
 *
 * Peli:    Sanassa on 5 kirjainta
 *
 * Peli:    Anna ensimminen kirjain:
 *
 * Pelaaja:S <ret>
 *
 * Peli:    Sanassa ei ollut yhtn S-kirjainta (-----)
 *  Vrien vastausten lukumr 1/9 (piirretn kumpu)
 *  Arvatut kirjaimet (S)
 *
 * Peli:    Anna seuraava kirjain:
 *
 * Pelaaja:T <ret>
 *
 * Peli:    Sanassa ei ollut yhtn T-kirjainta (-----)
 *  Vrien vastausten lukumr 2/9 (piirretn hirsipuun pylvs)
 *  Arvatut kirjaimet (S,T)
 *
 * Peli:    Anna seuraava kirjain:
 *
 * Pelaaja:O <ret>
 *
 * Peli:    Sanassa oli O-kirjain paikassa 2 (-O---)
 *  Anna seuraava kirjain
 * tai Peli:    Sanassa oli O-kirjain paikoissa 2,4 (-O-O-)
 * Pelaaja:P <ret>
 *
 * Peli:    Sanassa ei ollut P-kirjainta (-O---)
 *  Vrien vastausten lukumr 3/9 (piirretn poikkipuu)
 *  Anna seuraava kirjain
 *  Arvatut kirjaimet (S,T,P)
 *
 * Pelaaja:I <ret>
 *
 * Peli:    Sanassa oli I-kirjain paikassa 3 (-OI--)
 *  Anna seuraava kirjain
 *
 * Pelaaja:V <ret>
 *
 * Peli:    Sanassa ei ollut V-kirjainta (-OI---)
 *  Vrien vastausten lukumr 4/9 (piirretn kysi)
 *  Anna seuraava kirjain
 *  Arvatut kirjaimet (S,T,P,V)
 *
 * Pelaaja:K <ret>
 *
 * Peli:    Sanassa oli K-kirjain paikassa 1 (KOI--)
 *  Anna seuraava kirjain
 *
 * Pelaaja:U <ret>
 *
 * Peli:    Sanassa ei ollut U-kirjainta (KOI--)
 *  Vrien vastausten lukumr 5/9 (piirretn p)
 *  Anna seuraava kirjain
 *  Arvatut kirjaimet (S,T,P,V,U)
 *
 * Pelaaja:R <ret>
 *
 * Peli:    Sanassa oli R-kirjain paikassa 4 (KOIR-)
 *  Anna seuraava kirjain
 *
 * Pelaaja:A <ret>
 *
 * Peli:    Sanassa oli A-kirjain paikassa 5 (KOIRA)
 *  Onneksi olkoon voitit!
 *
 * Peli:    Haluatko jatkaa (Kyll/ei)
 *
 * Pelaaja:ei <ret>
 *
 * Peli pttyi.
 *
 * 
 * </pre>
 * 
 * @author vesal
 * @version 25.10.2010
 *
 */
public class Hirsipuu {

    private static final BasicShape[] hirrenOsat = {
        new Circle(0,0,1),
        new Line(0,1,0,7),
        new Line(0,7,3,7),
        new Line(3,7,3,6),
        new Circle(3,5.5,0.5),
        new Line(3,5,3,3),
        new Line(3,3,2,1),
        new Line(3,3,4,1),
        new Line(2,4.5,4,4.5)
     };
     

    /**
     * Tulostetaan ohjelman alkutervehdys    
     */
    public static void tervehdys() {
        System.out.println("Tervetuloa pelaamaan Hirsipuupeli!");
    }

    
    /**
     * Tulostetaan miss paikoissa oli oikeita kirjaimia.
     * Muoto: Sanassa oli S-kirjain paikoissa 3,4 (--SS-) 
     * @param kirjain tulostettava kirjain
     * @param paikat taulukko paikoisa joista kirjain on lytynyt
     * @param oikeat jono jossa on - viel arvaamattoman kirjaimen kohdalle 
     * TODO testaaminen???
     */
    public static void tulostaPaikat(char kirjain,int[] paikat, StringBuilder oikeat) {
        
        if ( paikat.length == 0 ) {
            System.out.printf("Sanassa ei ollut yhtn %c-kirjainta (%s)%n",kirjain, oikeat);
            return;
        }

        String paikkaTeksti = "paikassa";
        if ( paikat.length > 1 ) paikkaTeksti = "paikoissa";
        System.out.printf("Sanassa oli %c-kirjain %s ",kirjain,paikkaTeksti);
        String erotin = "";
        for (int paikka:paikat) {
            System.out.print(erotin+paikka);
            erotin = ",";
        }
        System.out.println(" (" + oikeat + ")");
    }


    /**
     * Tulostetaan arvatut kirjaimet muodossa (A,K,S)
     * @param arvatut tulostettavat kirjaimet
     * TODO testaaminen???
     */
    public static void tulostaArvatut(String arvatut) {
        System.out.print("Arvatut kirjaimet (");
        String erotin = "";

        for (int i=0; i<arvatut.length(); i++) {
            System.out.print(erotin+arvatut.charAt(i));
            erotin = ",";
        }

        System.out.println(")");
    }


    /**
     * Tutkii miss paikoissa on oikeita kirjaimia sanassa.
     * Muuttaa vastaavat kirjaimet oikeat jonossa.
     * Jos sama kirjain tulee uudelleen, palautetaan tyhj taulukko
     * Jos jonot ovat erimittaiset, palautetaan aina tyhj taulukko
     *
     * @param sana Tutkittava sana
     * @param kirjain kirjain jota etsitn
     * @param oikeat jono johon merkitn kirjaimet oikeille paikoilleen
     * @return taulukon oikeista paikoista, mahdollisesti tyhjn
     * @example
     * <pre name="test">
     * #import java.util.Arrays;
     *   StringBuilder oikeat = new StringBuilder("-----");
     *   int[] paikat = tutkiPaikat("KISSA",'A',oikeat,'-');
     *   paikat.length === 1; paikat[0] === 5;
     *   Arrays.toString(paikat) === "[5]";    oikeat.toString() === "----A";
     *   paikat = tutkiPaikat("KISSA",'X',oikeat,'-');
     *   Arrays.toString(paikat) === "[]";     oikeat.toString() === "----A";
     *   paikat = tutkiPaikat("KISSA",'S',oikeat,'-');
     *   Arrays.toString(paikat) === "[3, 4]"; oikeat.toString() === "--SSA";
     *   paikat = tutkiPaikat("KISSA",'A',oikeat,'-');
     *   Arrays.toString(paikat) === "[]";     oikeat.toString() === "--SSA";
     *   
     *   paikat = tutkiPaikat("KANA",'N',oikeat,'-');
     *   Arrays.toString(paikat) === "[]";     oikeat.toString() === "--SSA";
     *   paikat = tutkiPaikat("KISSAT",'T',oikeat,'-');
     *   Arrays.toString(paikat) === "[]";     oikeat.toString() === "--SSA";
     *   paikat = tutkiPaikat("KOIRA",'R',oikeat,'-');
     *   Arrays.toString(paikat) === "[]";     oikeat.toString() === "--SSA";
     *   paikat = tutkiPaikat("SASSA",'S',oikeat,'-');
     *   Arrays.toString(paikat) === "[1]";    oikeat.toString() === "S-SSA";
     *   oikeat.setCharAt(0,'-');
     *   paikat = tutkiPaikat("KISSA",'k',oikeat,'-');
     *   Arrays.toString(paikat) === "[1]";     oikeat.toString() === "K-SSA";
     *   paikat = tutkiPaikat("kissa",'I',oikeat,'-');
     *   Arrays.toString(paikat) === "[2]";     oikeat.toString() === "KISSA";
     * </pre>
     * 
     * @example
     * <pre name="test">
     *   StringBuilder oikeat;
     *   int[] paikat;
     *
     *   oikeat = new StringBuilder($oik1);
     *   paikat = tutkiPaikat($sana,$kirjain,oikeat,'-');
     *   Arrays.toString(paikat) === $tulos;     oikeat.toString() === $oik2;
     *   
     *      $oik1    | $sana    | $kirjain | $tulos      |  $oik2
     *     ---------------------------------------------------------
     *      "-----"  | "KISSA"  |  'A'     | "[5]"       |  "----A"  // Tavallinen tapaus, lytyy 1
     *      "----A"  | "KISSA"  |  'X'     | "[]"        |  "----A"  // ei lydy
     *      "----A"  | "KISSA"  |  'S'     | "[3, 4]"    |  "--SSA"  // lytyy 2 
     *      "--SSA"  | "KISSA"  |  'A'     | "[]"        |  "--SSA"  // merkki on jo
     *      "--SSA"  | "KANA"   |  'N'     | "[]"        |  "--SSA"  // sana lyhempi
     *      "--SSA"  | "KISSAT" |  'T'     | "[]"        |  "--SSA"  // sana pidempi
     *      "--SSA"  | "KOIRA"  |  'R'     | "[]"        |  "--SSA"  // oikeissa on jo merkki (vaikka eri)
     *      "--SSA"  | "SASSA"  |  'S'     | "[1]"       |  "S-SSA"  // jonot eptahdissa (sana eri tilanteesta kuin oikeat)
     *      "----A"  | "SASSA"  |  'S'     | "[1, 3, 4]" |  "S-SSA"  // lytyy kolme
     *      "--SSA"  | "KISSA"  |  'k'     | "[1]"       |  "K-SSA"  // eri case kirjaimessa
     *      "K-SSA"  | "kissa"  |  'I'     | "[2]"       |  "KISSA"  // eri case sanassa
     * 
     * </pre>
     * 
     */
    public static int[] tutkiPaikat(String sana,char kirjain, StringBuilder oikeat, char taytemerkki) {
        if ( sana.length() != oikeat.length() ) return new int[0];
        
        int[] paikat = new int[sana.length()];
        int lkm = 0; 
        char isokirjain = Character.toUpperCase(kirjain);
        
        for (int i = 0; i < sana.length(); i++) {
            char c = Character.toUpperCase(sana.charAt(i));
            if ( c == isokirjain && oikeat.charAt(i) == taytemerkki ) {
                paikat[lkm] = i+1;
                lkm++;
                oikeat.setCharAt(i, isokirjain);
            }
        }        
        /* 
        int[] palautettava = new int[lkm]; 
        for (int i=0; i<lkm; i++) palautettava[i] = paikat[i];  
        return palautettava;
        */
        return Arrays.copyOf(paikat, lkm);
    }


    /**
     * Pelataan yksi kierros hirsipuupeli
     * @param args ei kytss
     */
    public static void main(String[] args) {
        tervehdys();
        String nimi = Syotto.kysy("Anna nimesi");
        System.out.println("Terve " + nimi + "!");

        Window ikkuna = new Window(400,400);
        ikkuna.scale(0,0,10,10);
       
        ikkuna.showWindow();

        String sana = "kissat"; // TODO: arvo thn sana jostakin
        sana = sana.toUpperCase();

        System.out.printf("Sanassa on %d kirjainta%n",sana.length());
        
        int maxVaaria =hirrenOsat.length;    
        int vaaria = 0;
        String arvatut = "";
        StringBuilder oikeat = new StringBuilder(Mjonot.tayta("-", sana.length()));
        int n = 1;
        
        while ( true ) { // ikuinen silmukka
            char kirjain; 
            String vastaus = Syotto.kysy("Anna " + n + ". kirjain");
            if ( vastaus.length() == 0 ) continue;
            kirjain = Character.toUpperCase(vastaus.charAt(0));
            
            int[] paikat = tutkiPaikat(sana,kirjain,oikeat,'-');
            
            tulostaPaikat(kirjain,paikat,oikeat);
            
            if ( paikat.length == 0 ) {
                ikkuna.add(hirrenOsat[vaaria]);
                vaaria++;
                System.out.printf("Vrien vastausten lukumr %d/%d%n",vaaria,maxVaaria);
                if ( vaaria >= maxVaaria ) {
                    System.out.println("Hvisit!!!");
                    break;
                }
            }
            
            n++;
            arvatut += kirjain;
            tulostaArvatut(arvatut);
            
            if ( sana.equals(oikeat.toString()) ) {
                System.out.println("Onneksi olkoon voitit!");
                break;
            }
        }
        
    }

}
